const system = require("../system");
const moment = require('moment')
const settings = require("../../config/settings");
const md5 = require("MD5");
class AppServiceBase {
  constructor() {
    this.restClient = system.getObject("util.restClient");
    this.execClient = system.getObject("util.execClient");
    this.cacheManager = system.getObject("db.common.cacheManager");
    this.pushlogSve = system.getObject("service.common.pushlogSve");
    this.logCtl = system.getObject("service.common.oplogSve");
  }
  /**
   * 验证签名
   * @param {*} params 要验证的参数 
   * @param {*} app_key 应用的校验key
   */
  async verifySign(params, app_key) {
    if (!params) {
      return system.getResult(null, "请求参数为空");
    }
    if (!params.sign) {
      return system.getResult(null, "请求参数sign为空");
    }
    var signArr = [];
    var keys = Object.keys(params).sort();
    if (keys.length == 0) {
      return system.getResult(null, "请求参数信息为空");
    }
    for (let k = 0; k < keys.length; k++) {
      const tKey = keys[k];
      if (tKey != "sign" && params[tKey] && !(params[tKey] instanceof Array)) {
        signArr.push(tKey + "=" + params[tKey]);
      }
    }
    if (signArr.length == 0) {
      return system.getResult(null, "请求参数组装签名参数信息为空");
    }
    var resultSignStr = signArr.join("&") + "&key=" + app_key;
    var resultTmpSign = md5(resultSignStr).toUpperCase();
    if (params.sign != resultTmpSign) {
      return system.getResult(null, "返回值签名验证失败");
    }
    return system.getResultSuccess();
  }
  async restPostUrl(pobj, url) {
    var rtn = await this.restClient.execPost(pobj, url);
    if (!rtn || !rtn.stdout) {
      return system.getResult(null, "restPost data is empty");
    }
    var result = JSON.parse(rtn.stdout);
    return result;
  }
  async execPostUrl(pobj, url) {
    var rtn = await this.execClient.execPost(pobj, url);
    if (!rtn || !rtn.stdout) {
      return system.getResult(null, "execPost data is empty");
    }
    var result = JSON.parse(rtn.stdout);
    return result;
  }
  async opAliyunRpcVerifyParam(pobj) {//参数信息验证
    var verify = system.getResultSuccess();
    if (!pobj.interface_params) {
      verify = system.getResult(null, "interface_params can not be empty，100440");
    }
    var interface_params_info = JSON.parse(pobj.interface_params);
    if (!interface_params_info || !interface_params_info.action) {
      verify = system.getResult(null, "interface_params.action can not be empty，100443");
    }
    verify.data = interface_params_info;
    return verify;
  }
  /**
   * 阿里RPC调用
   * @param {*} pobj {action: rpcParam.action,reqbody: pobj.actionBody,rpcParam: rpcParam}
   * @param {*} params {为阿里的接口参数}
   */
  async opAliyunRpcReq(pobj, params) {
    try {
      // var action = obj.action;
      // var reqbody = obj.reqbody;
      // var rpcParam = obj.rpcParam;
      var aliyunClient = system.getObject("util.aliyunClient");
      var rtn = await aliyunClient.reqCustomByGet(params);
      return rtn;
    } catch (e) {
      var errorMsg = "error：" + e.stack;
      //日志记录
      this.logCtl.error({
        appid: pobj.appInfo ? pobj.appInfo.uapp_id || "" : "",
        appkey: pobj.appInfo ? pobj.appInfo.uapp_key || "" : "",
        requestId: pobj.requestId || "",
        op: "service/impl/utilsSve/opAliyunClientPost.js/opAliyunRpcReq",
        content: errorMsg,
        // clientIp: pobj.clientIp,
        optitle: pobj.opType + "推送操作异常->opAliyunRpcReq",
      });
      return system.getResult(null, errorMsg);
    }
  }
  /**
    * 阿里RPC调用-post请求
    * @param {*} pobj {action: rpcParam.action,reqbody: pobj.actionBody,rpcParam: rpcParam}
    * @param {*} params {为阿里的接口参数}
    */
  async opAliyunRpcReqByPost(pobj, params) {
    try {
      // var action = obj.action;
      // var reqbody = obj.reqbody;
      // var rpcParam = obj.rpcParam;
      var aliyunClient = system.getObject("util.aliyunClient");
      var rtn = await aliyunClient.reqCustomByPost(params);
      return rtn;
    } catch (e) {
      var errorMsg = "error：" + e.stack;
      //日志记录
      this.logCtl.error({
        appid: pobj.appInfo ? pobj.appInfo.uapp_id || "" : "",
        appkey: pobj.appInfo ? pobj.appInfo.uapp_key || "" : "",
        requestId: pobj.requestId || "",
        op: "service/impl/utilsSve/opAliyunClientPost.js/opAliyunRpcReqByPost",
        content: errorMsg,
        // clientIp: pobj.clientIp,
        optitle: pobj.opType + "推送操作异常->opAliyunRpcReqByPost",
      });
      return system.getResult(null, errorMsg);
    }
  }
  async disposePushResult(pobj, result, opTitleDesc) {//处理结果信息--内部使用
    if (result.status != 0) {
      this.pushlogSve.createFailLogDb({
        appid: pobj.appInfo.uapp_id,
        appkey: pobj.appInfo.uapp_key,
        requestId: pobj.requestId || "",
        content: JSON.stringify(pobj),//推送的参数信息
        resultInfo: JSON.stringify(result),
        clientIp: pobj.clientIp || "",
        failType: 2,
        opTitle: pobj.opType + "推送操作失败->" + opTitleDesc,
        pushNumber: pobj.pushNumber || 1
      });
      result.data = null;
      return result;
    }
    this.pushlogSve.createDb({
      appid: pobj.appInfo.uapp_id,
      appkey: pobj.appInfo.uapp_key,
      requestId: pobj.requestId || "",
      op: JSON.stringify(pobj.interface_info),//推送的接口信息
      content: JSON.stringify(pobj),//推送的参数信息
      resultInfo: JSON.stringify(result),
      returnType: '1',
      opTitle: pobj.opType + "数据推送成功->" + opTitleDesc
    });
    result.data = null;
    return result;
  }
  async disposePushResultFail(pobj, result, opTitleDesc) {//处理失败的结果信息--内部使用
    this.pushlogSve.createFailLogDb({
      appid: pobj.appInfo.uapp_id,
      appkey: pobj.appInfo.uapp_key,
      requestId: pobj.requestId || "",
      content: JSON.stringify(pobj),//推送的参数信息
      resultInfo: JSON.stringify(result),
      clientIp: pobj.clientIp || "",
      failType: 2,
      opTitle: pobj.opType + "推送操作失败->" + opTitleDesc,
      pushNumber: pobj.pushNumber || 1
    });
    result.data = null;
    return result;
  }
  async getAliossjavaFileUrl(pobj, params) {//上传ali oss 文件调用
    try {
      var aliOssFileInfo = await this.execPostUrl(params, settings.aliossjavaUrl());
      if (aliOssFileInfo) {
        var fileUrl = params.filedir + "/" + params.objectName;
        return system.getResultSuccess(fileUrl);
      }
      return system.getResult(null, "java ali oss upload fail");
    } catch (e) {
      this.logCtl.error({
        appid: pobj.appInfo ? pobj.appInfo.uapp_id || "" : "",
        appkey: pobj.appInfo ? pobj.appInfo.uapp_key || "" : "",
        requestId: pobj.requestId || "",
        op: "service/app.base.js/getAliossjavaFileUrl",
        content: "error：" + e.stack,
        // clientIp: pobj.clientIp,
        optitle: pobj.opType || "" + "aliossjava-upload阿里上传文件操作异常->getAliossjavaFileUrl",
      });
      return system.getResult(-200, "java ali oss upload error");
    }
  }
  /*
  返回20位业务订单号
  prefix：业务前缀
  */
  async getBusUid(prefix) {
    prefix = (prefix || "");
    if (prefix) {
      prefix = prefix.toUpperCase();
    }
    var prefixlength = prefix.length;
    var subLen = 8 - prefixlength;
    var uidStr = "";
    if (subLen > 0) {
      uidStr = await this.getUidInfo(subLen, 60);
    }
    var timStr = moment().format("YYYYMMDDHHmm");
    return prefix + timStr + uidStr;
  }
  /*
  len：返回长度
  radix：参与计算的长度，最大为62
  */
  async getUidInfo(len, radix) {
    var chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('');//长度62，到yz长度为长36
    var uuid = [], i;
    radix = radix || chars.length;
    if (len) {
      for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random() * radix];
    } else {
      var r;
      uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
      uuid[14] = '4';
      for (i = 0; i < 36; i++) {
        if (!uuid[i]) {
          r = 0 | Math.random() * 16;
          uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
        }
      }
    }
    return uuid.join('');
  }
}
module.exports = AppServiceBase;
