const AppServiceBase = require("../../app.base");
var settings = require("../../../../config/settings");
const system = require("../../../system");

//此类用来db中配置来进行调用-主要功能是推送阿里商标业务
class UtilsTmAliyunService extends AppServiceBase {
  constructor() {
    super();
    //ICON 图标图样，CARD 身份证，ATTORNEY　授权书ossKey，PASSPORT 护照，BUSINESS_LICENSE 营业执照，LEGAL_NOTICE 法律声明，SUPPLEMENT 补正图片的上传
    this.aliOssFileType = { ICON: "ICON", CARD: "CARD", ATTORNEY: "ATTORNEY", PASSPORT: "PASSPORT", BUSINESS_LICENSE: "BUSINESS_LICENSE", LEGAL_NOTICE: "LEGAL_NOTICE", SUPPLEMENT: "SUPPLEMENT" };
  }
  async tmCreateApplicant(pobj) {//创建商标申请人信息
    var verifyResult = await this.opAliyunRpcVerifyParam(pobj);
    var apply = pobj.actionBody.delivery_content.apply;
    var orderContact = pobj.actionBody.delivery_content.orderContact;
    var orderInfo = pobj.actionBody.order_info;
    var tmInfo = pobj.actionBody.delivery_content.tm;
    if (!orderInfo) {
      verifyResult = system.getResult(null, "orderInfo data is empty，100450");
    }
    if (Number(orderInfo.orderStatus) != 2) {
      verifyResult = system.getResult(null, "orderInfo status is error，100453");
    }
    if (!apply) {
      verifyResult = system.getResult(null, "apply data is empty，100455");
    }
    var rpcParam = verifyResult.data;
    if (!tmInfo) {
      return system.getResult(null, "tmInfo data is empty，100460");
    }
    var idCardOssKey = "";
    var businessLicenceOssKey = "";
    var legalNoticeOssKey = "";
    if (apply.customerType != "ent" && apply.identityCardPic) {
      idCardOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.CARD, apply.identityCardPic, rpcParam);
      if (!idCardOssKey) {
        verifyResult = system.getResult(null, "identityCardPic ali oss upload is error，100462");
      }
    }
    if (apply.customerType == "ent" && apply.businessLicensePic) {
      businessLicenceOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.BUSINESS_LICENSE, apply.businessLicensePic, rpcParam);
      if (!businessLicenceOssKey) {
        verifyResult = system.getResult(null, "businessLicensePic ali oss upload is error，100464");
      }
    }
    if (apply.smwjUrl) {
      legalNoticeOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.LEGAL_NOTICE, apply.smwjUrl, rpcParam);
      if (!legalNoticeOssKey) {
        verifyResult = system.getResult(null, "smwjUrl ali oss upload is error，100466");
      }
    }
    var picUrl = "";
    var authorizationOssKey = "";
    if (tmInfo.picUrl) {
      picUrl = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.ICON, tmInfo.picUrl, rpcParam);
      if (!picUrl) {
        verifyResult = system.getResult(null, "picUrl ali oss upload is error，100467");
      }
    }
    if (apply.gzwtsUrl) {
      authorizationOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.ATTORNEY, apply.gzwtsUrl, rpcParam);
      if (!authorizationOssKey) {
        verifyResult = system.getResult(null, "gzwtsUrl ali oss upload is error，100468");
      }
    }
    if (verifyResult.status != 0) {
      return await this.disposePushResultFail(pobj, verifyResult, "ali->tmCreateApplicant->verifyResult");
    }
    var actionBody = {
      ContactName: orderContact.contactName || "",//String 	否 	张三 	联系人姓名
      ContactNumber: orderContact.mobile || "",//String 	否 	18611112222 	联系人电话
      ContactEmail: orderContact.email || "",//String 	否 	test@alibaba.com 	联系人邮箱
      ContactZipcode: apply.zipCode || "",//String 	否 	100000 	联系人邮编
      ContactAddress: apply.applyAddr,//String 	否 	北京市朝阳区绿地中心A座 	联系人地址

      ApplicantType: apply.customerType && apply.customerType == "ent" ? 2 : 1,//Integer 	否 	1 	申请人类型 1, "个人"；2, "企业"
      ApplicantRegion: 1,//Integer 	否 	1 	阿拉伯数字 1, "中国大陆"；2, "中国香港"；3, "中国澳门"；4, "中国台湾"；5, "国外"			
      ApplicantName: apply.name,//String 	否 	张三 	申请人名称
      CardNumber: apply.customerType && apply.customerType == "ent" ? apply.code : apply.identityCardNo,//String 	否 	123456 	证件号码
      Address: apply.applyAddr,//String 	否 	北京市朝阳区绿地中心A座 	申请人地址
      IdCardOssKey: idCardOssKey,//String 	否 	1219541161213057/card/ujd8hthwfdk.png 	身份证ossKey
      BusinessLicenceOssKey: businessLicenceOssKey,//String 	否 	1219541161213057/business_license/gmzl6cvbk48.png 	营业执照ossKey
      LegalNoticeOssKey: legalNoticeOssKey//String 	否 	1219541161213057/legal_notice/mlnpsjmaed.jpg 	法律声明ossKey

      // Province: apply.,//String 	否 	安徽省 	省份
      // EName: apply.,//String 	否 	zhang san 	英文名称
      // EAddress: apply.,//String 	否 	bei jing shi 	英文地址
      // Country: apply.,//String 	否 	2096 	国家
      // PassportOssKey: apply.,//String 	否 	1219541161213057/pass_port/gmzl16cvbk48.png 	护照ossKey
      // AuthorizationOssKey: apply.,//String 	否 	1219541161213057/attorney/gy378w6btam.jpg 	授权书ossKey--联系了开发，创建申请人时不用填写，在创建商标时填写
    };
    var params = {
      action: rpcParam.action,
      reqbody: actionBody,
      rpcParam: rpcParam
    }
    // var applicantResult = await this.opAliyunRpcReq(pobj, params);
    // if (applicantResult.status != 0) {
    //   return await this.disposePushResultFail(pobj, applicantResult, "ali->tmCreateApplicant->applicantResult");
    // }//++++++++++++++++++++++323661
    var classificationsResult = await this.getClassifications(pobj, pobj.actionBody.delivery_content.nclones, rpcParam);
    if (Object.keys(classificationsResult).length == 0) {
      classificationsResult = system.getResult(null, "获取阿里尼斯数据为空，100469");
      return await this.disposePushResultFail(pobj, classificationsResult, "ali->tmCreateApplicant->classificationsResult");
    }
    var tmCreateResult = await this.tmCreate(pobj, tmInfo, rpcParam, picUrl, authorizationOssKey, classificationsResult, 323661);//applicantResult.data.ApplicantId);
    return await this.disposePushResult(pobj, tmCreateResult, "ali->tmCreateApplicant->tmCreateResult");
  }
  async bindAliTmApplicantGzwtsUrl(pobj) {//绑定商标申请人信息-(绑定授权书Oss文件)
    var verifyResult = await this.opAliyunRpcVerifyParam(pobj);
    if (!pobj.actionBody.applicantId) {
      verifyResult = system.getResult(null, "applicantId param is empty，100500");
    }
    if (!pobj.actionBody.gzwtsUrl) {
      verifyResult = system.getResult(null, "gzwtsUrl param is empty，100503");
    }
    var authorizationOssKey = "";
    if (pobj.actionBody.gzwtsUrl) {
      authorizationOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.ATTORNEY, pobj.actionBody.gzwtsUrl, rpcParam);
      if (!authorizationOssKey) {
        verifyResult = system.getResult(null, "gzwtsUrl ali oss upload is error，100505");
      }
    }
    if (verifyResult.status != 0) {
      return await this.disposePushResultFail(pobj, verifyResult, "ali->bindAliTmApplicantGzwtsUrl->verifyResult");
    }
    var params = {
      action: rpcParam.action,
      reqbody: { ApplicantId: pobj.actionBody.applicantId, AuthorizationOssKey: authorizationOssKey },
      rpcParam: rpcParam
    }
    var result = await this.opAliyunRpcReq(pobj, params);
    return await this.disposePushResult(pobj, result, "ali->bindAliTmApplicantGzwtsUrl->result");
  }
  async updateAliTmApplicant(pobj) {//更新商标申请人信息
    var verifyResult = await this.opAliyunRpcVerifyParam(pobj);
    var apply = pobj.actionBody.delivery_content.apply;
    var orderContact = pobj.actionBody.delivery_content.orderContact;
    if (!apply) {
      verifyResult = system.getResult(null, "apply data is empty，100510");
    }
    var idCardOssKey = "";
    var businessLicenceOssKey = "";
    var legalNoticeOssKey = "";
    if (apply.customerType != "ent" && apply.identityCardPic) {
      idCardOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.CARD, apply.identityCardPic, rpcParam);
      if (!idCardOssKey) {
        verifyResult = system.getResult(null, "identityCardPic ali oss upload is error，100512");
      }
    }
    if (apply.customerType == "ent" && apply.businessLicensePic) {
      businessLicenceOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.BUSINESS_LICENSE, apply.businessLicensePic, rpcParam);
      if (!businessLicenceOssKey) {
        verifyResult = system.getResult(null, "businessLicensePic ali oss upload is error，100514");
      }
    }
    if (apply.smwjUrl) {
      legalNoticeOssKey = await this.getAliOssFileUrl("trade-mark-user-upload", pobj, this.aliOssFileType.LEGAL_NOTICE, apply.smwjUrl, rpcParam);
      if (!legalNoticeOssKey) {
        verifyResult = system.getResult(null, "smwjUrl ali oss upload is error，100516");
      }
    }
    if (verifyResult.status != 0) {
      return await this.disposePushResultFail(pobj, verifyResult, "ali->updateAliTmApplicant->verifyResult");
    }
    var rpcParam = verify.data;
    var actionBody = {
      ApplicantId: pobj.actionBody.applicantId,//Long 	是 	123456 	申请人ID
      ContactName: orderContact.contactName || "",//String 	否 	张三 	联系人姓名
      ContactNumber: orderContact.mobile || "",//String 	否 	18611112222 	联系人电话
      ContactEmail: orderContact.email || "",//String 	否 	test@alibaba.com 	联系人邮箱
      ContactZipcode: apply.zipCode || "",//String 	否 	100000 	联系人邮编
      ContactAddress: apply.applyAddr,//String 	否 	北京市朝阳区绿地中心A座 	联系人地址

      ApplicantType: apply.customerType && apply.customerType == "ent" ? 2 : 1,//Integer 	否 	1 	申请人类型 1, "个人"；2, "企业"
      ApplicantRegion: 1,//Integer 	否 	1 	阿拉伯数字 1, "中国大陆"；2, "中国香港"；3, "中国澳门"；4, "中国台湾"；5, "国外"			
      ApplicantName: apply.name,//String 	否 	张三 	申请人名称
      CardNumber: apply.customerType && apply.customerType == "ent" ? apply.code : apply.identityCardNo,//String 	否 	123456 	证件号码
      Address: apply.applyAddr,//String 	否 	北京市朝阳区绿地中心A座 	申请人地址
      IdCardOssKey: idCardOssKey,//String 	否 	1219541161213057/card/ujd8hthwfdk.png 	身份证ossKey
      BusinessLicenceOssKey: businessLicenceOssKey,//String 	否 	1219541161213057/business_license/gmzl6cvbk48.png 	营业执照ossKey
      LegalNoticeOssKey: legalNoticeOssKey//String 	否 	1219541161213057/legal_notice/mlnpsjmaed.jpg 	法律声明ossKey

      // Province: apply.,//String 	否 	安徽省 	省份
      // EName: apply.,//String 	否 	zhang san 	英文名称
      // EAddress: apply.,//String 	否 	bei jing shi 	英文地址
      // Country: apply.,//String 	否 	2096 	国家
      // PassportOssKey: apply.,//String 	否 	1219541161213057/pass_port/gmzl16cvbk48.png 	护照ossKey
      // AuthorizationOssKey: apply.,//String 	否 	1219541161213057/attorney/gy378w6btam.jpg 	授权书ossKey--联系了开发，创建申请人时不用填写，在创建商标时填写
    };
    var params = {
      action: rpcParam.action,
      reqbody: actionBody,
      rpcParam: rpcParam
    }
    var result = await this.opAliyunRpcReq(pobj, params);
    return await this.disposePushResult(pobj, result, "ali->updateAliTmApplicant->result");
  }
  async closeAliTrademarkApplication(pobj) {//主动关闭商标申请
    var verifyResult = await this.opAliyunRpcVerifyParam(pobj);
    if (!pobj.actionBody.parentCode) {
      verifyResult = system.getResult(null, "parentCode param is empty，100490");
    }
    if (verifyResult.status != 0) {
      return await this.disposePushResultFail(pobj, verifyResult, "ali->closeAliTrademarkApplication->verifyResult");
    }
    var rpcParam = verifyResult.data;
    var params = {
      action: rpcParam.action,
      reqbody: { ParentCode: pobj.actionBody.parentCode },
      rpcParam: rpcParam
    }
    var result = await this.opAliyunRpcReq(pobj, params);
    return await this.disposePushResult(pobj, result, "ali->closeAliTrademarkApplication->result");
  }

  //--------------------------------------------------内部辅助方法-------------------start-----------------

  async tmCreate(pobj, tmInfo, rpcParam, picUrl, authorizationOssKey, classifications, applicantId) {//创建商标申请信息-内部使用－配合tmCreateApplicant方法使用
    // 商标类型形式： 3: 字, 4: 图, 5: 字图
    var trademarkNameType = 1;
    if (tmInfo.tmFormType == "4") {
      trademarkNameType = 2;
    }
    if (tmInfo.tmFormType == "5") {
      trademarkNameType = 3;
      tmInfo.tmName = tmInfo.tmName + "及图";
    }
    var actionBody = {
      AutoPay: true,
      TrademarkNameType: trademarkNameType,//String 	否 	1 	商标申请类型 1-文字 2-图形 3-文字及图形
      TrademarkName: tmInfo.tmName,//String 	是 	pop-test 	商标名称
      TrademarkComment: tmInfo.tmName,//String 	否 	测试 	商标说明  
      TrademarkIcon: picUrl,//String 	是 	1219541161213057/icon/gii9s4oyg8.jpg 	商标对应图案的ossKey,有彩色图标请填写彩色图标，没有请填写黑白图标
      BlackAndWhiteIcon: picUrl,//String 	否 	1219541161213057/icon/gii9s4oyg8.jpg
      ApplicationType: 1,//Integer 	否 	1 	固定值1
      ApplicantId: applicantId,//String 	否 	111 	申请人ID，通过申请人API创建申请人获得，或者通过申请人列表查询接口获得
      Classifications: JSON.stringify(classifications),//String 	是 	{"38":"380005,380024,380050,380040,38020148,380004,38010007,38020149,38020141,380053"} 	商标分类

      AuthorizationOssKey: authorizationOssKey,//String 	否 	1219541161213057/attorney/9cgvhk2yd1l.JPG 	授权书ossKey
      Channel: "aliTmH5"//String 	否 	pop接口测试 	通道，调用方填写，独立标志，用于区分来源－自己随便定义
    };
    var params = {
      action: "CreateTrademarkApplication",
      reqbody: actionBody,
      rpcParam: rpcParam
    }
    // var tmCreateReslut = await this.opAliyunRpcReqByPost(pobj, params);
    var tmCreateReslut = await this.opAliyunRpcReq(pobj, params);
    return tmCreateReslut;
  }
  async getAliOssFileUrl(bucketName, pobj, fileType, selfFileUrl, rpcParam) {
    var fileUrl = "";
    var resultInfo = await this.getAliOssInfo(pobj, fileType, rpcParam);
    if (resultInfo.status != 0) {
      return fileUrl;
    }
    let lastHostLength = resultInfo.data.Host.lastIndexOf('/');
    let lastFileLength = selfFileUrl.lastIndexOf('/');
    var params = {
      "ossurl": selfFileUrl,
      "endpoint": "https:" + resultInfo.data.Host.substring(0, lastHostLength),
      "accessKeyId": resultInfo.data.AccessId,
      "bucketName": bucketName,//"trade-mark-user-upload",
      "filedir": resultInfo.data.FileDir,
      "objectName": selfFileUrl.substring(lastFileLength + 1, selfFileUrl.length),
      "encodePolicy": resultInfo.data.EncodedPolicy,
      "signature": resultInfo.data.Signature
    }
    var ossFileResult = await this.getAliossjavaFileUrl(pobj, params);
    if (ossFileResult.status == 0) {
      fileUrl = ossFileResult.data;
    }
    return fileUrl;
  }
  async getAliOssInfo(pobj, fileType, rpcParam) {
    var params = {
      action: "GenerateUploadFilePolicy",
      reqbody: { FileType: fileType },
      rpcParam: rpcParam
    }
    var resultInfo = await this.opAliyunRpcReq(pobj, params);
    return resultInfo;
  }
  async getClassifications(pobj, nclones, rpcParam) {
    var self = this;
    var aliNclThreesList = {};
    var aliNclTwosList = [];
    for (let i = 0; i < nclones.length; i++) {
      const nclOneItem = nclones[i];
      var nclThreeList = nclOneItem.nclThree;
      var getAliThreeList = [];
      for (let j = 0; j < nclThreeList.length; j++) {
        const nclThreeItem = nclThreeList[j];
        var aliThreeList = null;
        var aliNclTwosListIndex = aliNclTwosList.findIndex(f => f.pcode === nclThreeItem.pcode);
        if (aliNclTwosListIndex < 0) {
          var aliNclDataResult = await self.getAliNclInfo(pobj, nclThreeItem.pcode, rpcParam);
          if (aliNclDataResult.status == 0) {
            aliThreeList = aliNclDataResult.data;
            aliNclTwosList.push({
              pcode: nclThreeItem.pcode,
              nclThree: aliThreeList
            });
          }
        }//获取ali 的信息
        else {
          aliThreeList = aliNclTwosList[aliNclTwosListIndex].nclThree;
        }
        if (!aliThreeList) {
          continue;
        }
        for (let k = 0; k < aliThreeList.length; k++) {
          const aliNclThree = aliThreeList[k];
          if (aliNclThree.ClassificationName == nclThreeItem.name) {
            getAliThreeList.push(aliNclThree.ClassificationCode);
            break;
          }
        }
      }//一个大类下小项组装完毕
      aliNclThreesList[nclOneItem.code] = getAliThreeList.join(',');
      // aliNclThreesList["\"" + nclOneItem.code + "\""] = "\"" + getAliThreeList.join(',') + "\"";
    }
    return aliNclThreesList;
  }
  async getAliNclInfo(pobj, parentCode, rpcParam) {
    var params = {
      action: "ListClassifications",
      reqbody: { ParentCode: parentCode },
      rpcParam: rpcParam
    }
    var nclInfoResult = await this.opAliyunRpcReq(pobj, params);
    if (nclInfoResult.status != 0 || !nclInfoResult.data || nclInfoResult.data.TotalCount == 0) {
      return system.getResult(null, "ali ncl is empty，100465")
    }
    return system.getResultSuccess(nclInfoResult.data.Classifications.Classification);
  }

  //--------------------------------------------------内部辅助方法-------------------end-----------------
}
module.exports = UtilsTmAliyunService;
